import { Request, Response, NextFunction } from 'express';
import { validationResult } from 'express-validator';
import bcrypt from 'bcrypt';
import jwt from 'jsonwebtoken';
import { Pool } from 'pg';
import pool from '../config/database';
import { AppError } from '../middleware/error.middleware';
import { UserRegistrationData, UserResponse } from '../types/user.types';
import 'express-session';

declare module 'express-session' {
  interface SessionData {
    userId: string;
    userEmail: string;
    userType: string;
  }
}

// Interface pour les données de connexion
interface LoginData {
  email: string;
  password: string;
}

// Interface pour l'utilisateur en base de données
interface DBUser {
  id: number;
  email: string;
  password_hash: string;
  artist_name: string;
  user_type: string;
  bio_short?: string;
  bio_long?: string;
  profile_picture_url?: string;
  created_at: Date;
}

export const register = async (req: Request, res: Response): Promise<void> => {
  try {
    // Validation des données
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      res.status(400).json({ errors: errors.array() });
      return;
    }

    const { email, password, artist_name, user_type = 'artist', bio_short, bio_long }: UserRegistrationData = req.body;

    // Vérification si l'email existe déjà
    const existingUser = await pool.query(
      'SELECT id FROM users WHERE email = $1',
      [email]
    );

    if (existingUser.rows.length > 0) {
      res.status(409).json({
        error: 'Un utilisateur avec cette adresse email existe déjà'
      });
      return;
    }

    // Hachage du mot de passe
    const saltRounds = 12;
    const password_hash = await bcrypt.hash(password, saltRounds);

    // Insertion du nouvel utilisateur
    const result = await pool.query(
      `INSERT INTO users (
        email,
        password_hash,
        artist_name,
        user_type,
        bio_short,
        bio_long
      ) VALUES ($1, $2, $3, $4, $5, $6)
      RETURNING id, email, artist_name, user_type, bio_short, bio_long, profile_picture_url, created_at`,
      [email, password_hash, artist_name, user_type, bio_short, bio_long]
    );

    // Construction de la réponse
    const user: UserResponse = result.rows[0];

    // Envoi de la réponse
    res.status(201).json({
      message: 'Inscription réussie',
      user
    });

  } catch (error) {
    console.error('Erreur lors de l\'inscription:', error);
    res.status(500).json({
      error: 'Une erreur est survenue lors de l\'inscription'
    });
  }
};

/**
 * Contrôleur pour la connexion d'un utilisateur
 * @route POST /api/auth/login
 */
export const login = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
  try {
    // Validation des données
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      throw new AppError(400, 'Données de connexion invalides', errors.array());
    }

    const { email, password } = req.body;

    // Recherche de l'utilisateur dans la base de données
    const result = await pool.query<DBUser>(
      'SELECT * FROM users WHERE email = $1',
      [email]
    );

    const user = result.rows[0];

    // Vérification si l'utilisateur existe
    if (!user) {
      throw new AppError(401, 'Email ou mot de passe incorrect');
    }

    // Vérification du mot de passe
    const isPasswordValid = await bcrypt.compare(password, user.password_hash);
    if (!isPasswordValid) {
      throw new AppError(401, 'Email ou mot de passe incorrect');
    }

    // Création de la session
    req.session.userId = user.id.toString();
    req.session.userEmail = user.email;
    req.session.userType = user.user_type;

    // Création du token JWT pour une double sécurité
    const token = jwt.sign(
      { 
        userId: user.id,
        email: user.email,
        userType: user.user_type 
      },
      process.env.JWT_SECRET || 'votre_secret_par_defaut',
      { expiresIn: '24h' }
    );

    // Suppression du mot de passe avant d'envoyer la réponse
    const { password_hash, ...userWithoutPassword } = user;

    // Envoi de la réponse
    res.json({
      success: true,
      message: 'Connexion réussie',
      token,
      user: userWithoutPassword
    });

  } catch (error) {
    next(error);
  }
};

/**
 * Contrôleur pour la déconnexion d'un utilisateur
 * @route POST /api/auth/logout
 */
export const logout = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
  try {
    if (req.session) {
      // Destruction de la session
      req.session.destroy((err) => {
        if (err) {
          throw new AppError(500, 'Erreur lors de la déconnexion');
        }
        
        // Suppression du cookie de session
        res.clearCookie('wemusic.sid', {
          path: '/',
          httpOnly: true,
          secure: process.env.NODE_ENV === 'production',
          sameSite: process.env.NODE_ENV === 'production' ? ('strict' as const) : ('lax' as const)
        });
        
        // Envoi de la réponse
        res.json({
          success: true,
          message: 'Déconnexion réussie'
        });
      });
    } else {
      res.json({
        success: true,
        message: 'Déjà déconnecté'
      });
    }
  } catch (error) {
    next(error);
  }
}; 