-- Création du type ENUM pour user_type
CREATE TYPE user_type AS ENUM ('artist', 'venue_owner', 'partner', 'admin');

-- Création de l'extension uuid-ossp si elle n'existe pas
CREATE EXTENSION IF NOT EXISTS "uuid-ossp";

-- Création de la table users
CREATE TABLE users (
    -- Identifiant unique de l'utilisateur
    id UUID DEFAULT uuid_generate_v4() PRIMARY KEY,
    
    -- Informations d'authentification
    email TEXT NOT NULL UNIQUE,
    password_hash TEXT NOT NULL,
    
    -- Informations de profil
    artist_name TEXT NOT NULL,
    user_type user_type NOT NULL DEFAULT 'artist',
    
    -- Champs de biographie
    bio_short TEXT,
    bio_long TEXT,
    
    -- URL de la photo de profil
    profile_picture_url TEXT,
    
    -- Horodatage
    created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP
);

-- Création d'un index sur l'email pour optimiser les recherches
CREATE INDEX idx_users_email ON users(email);

-- Fonction pour mettre à jour automatiquement updated_at
CREATE OR REPLACE FUNCTION update_updated_at_column()
RETURNS TRIGGER AS $$
BEGIN
    NEW.updated_at = CURRENT_TIMESTAMP;
    RETURN NEW;
END;
$$ language 'plpgsql';

-- Trigger pour mettre à jour automatiquement updated_at
CREATE TRIGGER update_users_updated_at
    BEFORE UPDATE ON users
    FOR EACH ROW
    EXECUTE FUNCTION update_updated_at_column();

-- Commentaires sur la table et les colonnes
COMMENT ON TABLE users IS 'Table stockant les informations des utilisateurs de WeMusic';
COMMENT ON COLUMN users.id IS 'Identifiant unique de l''utilisateur';
COMMENT ON COLUMN users.email IS 'Adresse email unique de l''utilisateur, utilisée pour l''authentification';
COMMENT ON COLUMN users.password_hash IS 'Hash du mot de passe de l''utilisateur (à générer avec bcrypt)';
COMMENT ON COLUMN users.artist_name IS 'Nom d''artiste ou nom d''utilisateur';
COMMENT ON COLUMN users.user_type IS 'Type d''utilisateur (artist, venue_owner, partner, admin)';
COMMENT ON COLUMN users.bio_short IS 'Courte biographie de l''utilisateur';
COMMENT ON COLUMN users.bio_long IS 'Biographie détaillée de l''utilisateur';
COMMENT ON COLUMN users.profile_picture_url IS 'URL de la photo de profil';
COMMENT ON COLUMN users.created_at IS 'Date et heure de création du compte';
COMMENT ON COLUMN users.updated_at IS 'Date et heure de la dernière modification du compte'; 