import express, { Response as ExpressResponse } from 'express';
import cors from 'cors';
import morgan from 'morgan';
import dotenv from 'dotenv';
import session from 'express-session';
import cookieParser from 'cookie-parser';
import crypto from 'crypto';
import authRoutes from './routes/auth.routes';
import { errorHandler, AppError } from './middleware/error.middleware';
import { 
  limiter, 
  authLimiter, 
  securityMiddleware, 
  validateJson 
} from './middleware/security.middleware';

// Configuration des variables d'environnement
dotenv.config();

const app = express();
const port = process.env.PORT || 3001;
const nodeEnv = process.env.NODE_ENV || 'development';
const isProduction = nodeEnv === 'production';

// Génération d'un secret de session aléatoire si non défini
if (!process.env.SESSION_SECRET) {
  process.env.SESSION_SECRET = crypto.randomBytes(32).toString('hex');
  console.warn('⚠️  Attention : SESSION_SECRET non défini dans les variables d\'environnement.');
  console.warn('🔑 Un secret temporaire a été généré pour cette session.');
}

// Middlewares de sécurité
app.use(securityMiddleware);
app.use(limiter);

// Configuration CORS avec support des cookies
app.use(cors({
  origin: process.env.CORS_ORIGIN || 'http://localhost:3000',
  credentials: true,
  methods: ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS'],
  allowedHeaders: ['Content-Type', 'Authorization']
}));

// Configuration des cookies et sessions
app.use(cookieParser());

// Configuration de la session
const sessionConfig = {
  secret: process.env.SESSION_SECRET!,
  name: 'wemusic.sid', // Pour masquer l'utilisation d'express-session
  resave: false,
  saveUninitialized: false,
  rolling: true, // Renouvelle le cookie à chaque requête
  cookie: {
    secure: isProduction,
    httpOnly: true,
    maxAge: 60 * 60 * 1000, // 1 heure
    sameSite: isProduction ? ('strict' as const) : ('lax' as const),
    path: '/',
    domain: undefined // Le domaine sera automatiquement défini normalement
  }
};

// Avertissement en développement pour le cookie non sécurisé
if (!isProduction) {
  console.warn('⚠️  Mode développement : Cookie secure désactivé');
  console.warn('🔒 En production, activez HTTPS et secure: true');
}

app.use(session(sessionConfig));

// Middleware de base
app.use(morgan(nodeEnv === 'development' ? 'dev' : 'combined'));
app.use(express.json({
  verify: (req, res: ExpressResponse, buf, encoding) => {
    try {
      JSON.parse(buf.toString());
    } catch (e) {
      res.status(400).json({ message: 'JSON invalide dans la requête' });
      throw new Error('JSON invalide');
    }
  }
}));
app.use(express.urlencoded({ extended: true }));

// Middleware pour renouveler la session
app.use((req, res, next) => {
  if (req.session && req.session.userId) {
    // Renouvelle la session si l'utilisateur est authentifié
    req.session.touch();
  }
  next();
});

// Routes d'authentification avec rate limiting
app.use('/api/auth', authLimiter, authRoutes);

// Route de base de l'API
app.get('/api', (req, res) => {
  res.json({ 
    message: 'Bienvenue sur l\'API WeMusic!',
    environment: nodeEnv,
    version: '1.0.0'
  });
});

// Middleware de gestion des erreurs
app.use((err: Error | AppError, req: express.Request, res: express.Response, next: express.NextFunction) => {
  errorHandler(err, req, res, next);
});

// Gestion des routes non trouvées
app.use((req, res) => {
  res.status(404).json({ message: 'Route non trouvée' });
});

// Démarrage du serveur
app.listen(port, () => {
  console.log(`🚀 Serveur ${nodeEnv} démarré sur le port ${port}`);
  console.log(`👉 CORS activé pour: ${process.env.CORS_ORIGIN || 'http://localhost:3000'}`);
  if (!isProduction) {
    console.log('⚙️  Mode développement actif');
  }
}); 